@@ -1,5 +1,5 @@
 import pickle
-from typing import Optional, List
+from typing import Optional, List, Dict
 
 import ipywidgets as widgets
 import k3d
@@ -46,6 +46,16 @@ class SkeletonVisualizer:
         self.skeletons: List[DrawableSkeleton] = []
         self.joint_names_visible = widgets.Checkbox(description="Show Joint Names")
 
+    def visualize(self, skeletons: np.ndarray, colors: List[Color] = None, include_names: bool = False) -> None:
+        self.__assert_skeleton_shapes(skeletons)
+        colors = self.__init_colors(skeletons.shape[0], colors)
+        self.plot = self.__create_skeleton_plot(skeletons, colors, include_names)
+        self.plot.display()
+        if include_names:
+            self.__link_joint_name_visibility_with_checkbox()
+            display(self.joint_names_visible)
+        self.__display_color_changer()
+
     def visualize_video_from_file(
             self, file_name: str, colors: List[Color] = None,
             fps: int = 15, include_names: bool = False) -> None:
@@ -58,34 +68,57 @@ class SkeletonVisualizer:
             self, frames: np.ndarray, colors: List[Color] = None,
             fps: int = 15, include_names: bool = False) -> None:
         assert len(frames.shape) == 4
-        first_frame = frames[0]
-        self.visualize(first_frame, colors, include_names)
-        self.__display_video_player(fps, frames)
+        self.__assert_skeleton_shapes(frames[0])
+        colors = self.__init_colors(frames[0].shape[0], colors)
+
+        self.plot = k3d.plot(camera_auto_fit=False)
+        self.__adjust_camera_starting_position(frames[0], self.plot)
+        skeleton_part_size = self.__calculate_skeleton_part_size(frames[0])
+
+        frames = np.swapaxes(frames, 0, 1)
+        skeleton_timestamps: List[Dict[str, np.ndarray]] = [
+            {
+                str(timestamp_index): frames[current_frame_index][timestamp_index]
+                for timestamp_index in range(len(frames[current_frame_index]))
+            }
+            for current_frame_index in range(len(frames))
+        ]
+
+        for skeleton in map(
+                lambda skeleton_color_tuple: Skeleton(
+                    joint_positions=skeleton_color_tuple[0],
+                    joint_set=self.joint_set,
+                    part_size=skeleton_part_size,
+                    color=skeleton_color_tuple[1]),
+                zip(skeleton_timestamps, colors)):
+            drawable_skeleton = skeleton.to_drawable_skeleton_for_video()
+            self.plot += drawable_skeleton
+            self.skeletons.append(drawable_skeleton)
 
-    def visualize(self, skeletons: np.ndarray, colors: List[Color] = None, include_names: bool = False) -> None:
-        self.__assert_skeleton_shapes(skeletons)
-        colors = self.__init_colors(skeletons.shape[0], colors)
-        self.plot = self.__create_skeleton_plot(skeletons, colors, include_names)
         self.plot.display()
-        if include_names:
-            self.__link_joint_name_visibility_with_checkbox()
-            display(self.joint_names_visible)
+        self.__display_video_player(fps, frames)
 
     def __display_video_player(self, fps, frames):
         video_player: Play = Play(
             value=0,
-            min=0, max=frames.shape[0] - 1,
+            min=0, max=frames.shape[1] - 1,
             step=1, interval=1000 / fps,
             description='Press play', disabled=False
         )
+
         frame_updater = FrameUpdater(self.skeletons, frames, self.joint_set, video_player)
-        video_player.observe(frame_updater.update_plot, names='value')
+
         frame_slider = widgets.IntSlider(value=0, min=0, max=frames.shape[0] - 1, step=1, description='Frame')
+
         next_frame_button = widgets.Button(description='Next frame')
         next_frame_button.on_click(frame_updater.update_to_next_frame)
+
         previous_frame_button = widgets.Button(description='Prev frame')
         previous_frame_button.on_click(frame_updater.update_to_previous_frame)
+
         widgets.jslink((video_player, 'value'), (frame_slider, 'value'))
+        widgets.jslink((video_player, 'value'), (self.plot, 'time'))
+
         display(widgets.HBox([video_player, previous_frame_button, frame_slider, next_frame_button]))
 
     @staticmethod
@@ -106,12 +139,11 @@ class SkeletonVisualizer:
     def __create_skeleton_plot(self, skeletons: np.ndarray,
                                colors: List[Color], include_names: bool = False) -> Optional[Plot]:
         skeleton_part_size = self.__calculate_skeleton_part_size(skeletons)
-        skeleton_plot = k3d.plot(antialias=0, background_color=0xDDDDDD, camera_auto_fit=False)
-        centroid: np.ndarray = self.__calculate_centroid(skeletons)
-        skeleton_plot.camera = [0, 0, 0, centroid[0], centroid[1], centroid[2], 0, -1, 0]
+        skeleton_plot = k3d.plot(camera_auto_fit=False)
+        self.__adjust_camera_starting_position(skeletons, skeleton_plot)
         for skeleton in map(
                 lambda skeleton_color_tuple: Skeleton(
-                    joint_coordinates=skeleton_color_tuple[0],
+                    joint_positions=skeleton_color_tuple[0],
                     joint_set=self.joint_set,
                     part_size=skeleton_part_size,
                     color=skeleton_color_tuple[1]),
@@ -128,14 +160,27 @@ class SkeletonVisualizer:
         max_values = [abs(skeleton).max() for skeleton in skeletons]
         return (min(max_values) / 100.0) * self.size_scalar
 
-    def __calculate_centroid(self, skeletons: np.ndarray) -> np.ndarray:
-        return np.average(np.average(skeletons, axis=0), axis=0)
+    def __adjust_camera_starting_position(self, skeletons: np.ndarray, skeleton_plot: Plot) -> None:
+        centroid: np.ndarray = np.average(np.average(skeletons, axis=0), axis=0)
+        camera_up_vector: np.ndarray = np.zeros(shape=(3,))
+        for line_indices in self.joint_set.vertically_aligned_line_indices:
+            camera_up_vector += np.sum(
+                skeletons[:, line_indices[1]] - skeletons[:, line_indices[0]],
+                axis=0)
+        camera_up_vector /= np.linalg.norm(camera_up_vector, ord=2)
+        skeleton_plot.camera = [
+            0, 0, 0,
+            centroid[0], centroid[1], centroid[2],
+            camera_up_vector[0], camera_up_vector[1], camera_up_vector[2]]
 
     def __link_joint_name_visibility_with_checkbox(self) -> None:
         for skeleton in self.skeletons:
             for joint_name in skeleton.joint_names:
                 widgets.jslink((joint_name, 'visible'), (self.joint_names_visible, 'value'))
 
+    def __display_color_changer(self):
+        pass
+
 
 class MuPoTSVisualizer(SkeletonVisualizer):
     def __init__(self, size_scalar: float = 1.0):
